function [net1, net2, net3, opts] = train_model_discriminative(opts, net1, net2, net3, net4, imdb, getBatch)

opts.batchSize = 100 ;
opts.numSubBatches = 1 ;
opts.train = [] ;
opts.val = [] ;
opts.numEpochs = 300 ;
opts.gpus = [] ; % which GPU devices to use (none, one, or more)
opts.learningRate = 0.001 ;% 0.001
opts.continue = false ;
opts.conserveMemory = false ;
opts.backPropDepth = +inf ;
opts.sync = false ;
opts.cudnn = true ;
opts.weightDecay = 0.0005 ; % 0.005
opts.momentum = 0.9 ;
opts.errorFunction = 'multiclass' ;
opts.errorLabels = {} ;

% learningRate_array = repmat(opts.learningRate , max(1,floor(opts.numEpochs / length(opts.learningRate))),1); %logspace(-2, -4, 60) ones(1,60, 'single')
% learningRate_array = reshape(learningRate_array, 1, []);
 
if isempty(opts.train), opts.train = find(imdb.images.set==1) ; end
if isempty(opts.val), opts.val = find(imdb.images.set==3) ; end
if isnan(opts.train), opts.train = [] ; end

opts.batch_size = min(opts.batch_size, numel(opts.train));

net1 = initialize_momentum(net1);
net2 = initialize_momentum(net2);
net3 = initialize_momentum(net3);
net4 = initialize_momentum(net4);

opts.errorFunction = @error_multiclass ;

interval = 10;

% -------------------------------------------------------------------------
%                           Train and validate
% -------------------------------------------------------------------------
evaluateMode = false;
model_file = [opts.working_folder, 'iter_',...
    num2str(opts.numEpochs) ,'_model.mat'];

if exist(model_file, 'file') && opts.force_learn == false;
    load(model_file);
else
    for epoch=1:opts.numEpochs
        fprintf('Iteration %d / %d\n', epoch, opts.numEpochs);
        
        learningRate = opts.learningRate(min(epoch, numel(opts.learningRate)));
        train = opts.train
%         train = randsample(opts.train, numel(opts.train));
        val = opts.val ;
        
        [net1, net2, net3, net4, stats.train] = process_epoch_discriminative(opts, getBatch, epoch, train, learningRate, imdb, net1, net2, net3, net4) ;
        [~,~,~,~,stats.val] = process_epoch_discriminative(opts, getBatch, epoch, val, 0, imdb, net1, net2, net3, net4) ;
        
        sets = {'train', 'val'};
        for f = sets
            f = char(f) ;
            n = numel(eval(f)) ;
            info.(f).speed(epoch) = n / stats.(f)(1) * max(1, 1) ;
            info.(f).objective(epoch) = stats.(f)(2) / n ;
            info.(f).error(:,epoch) = stats.(f)(3:end) / n ;
        end
        figure(1) ; clf ;
        hasError = isa(opts.errorFunction, 'function_handle') ;
        subplot(1,1+hasError,1) ;
        if ~evaluateMode
            semilogy(1:epoch, info.train.objective, '.-', 'linewidth', 2) ;
            hold on ;
        end
        semilogy(1:epoch, info.val.objective, '.--') ;
        xlabel('training epoch') ; ylabel('energy') ;
        grid on ;
        h=legend(sets) ;
        set(h,'color','none');
        title('objective') ;
        if hasError
            subplot(1,2,2) ; leg = {} ;
            if ~evaluateMode
                plot(1:epoch, info.train.error', '.-', 'linewidth', 2) ;
                hold on ;
                leg = horzcat(leg, strcat('train ', opts.errorLabels)) ;
            end
            plot(1:epoch, info.val.error', '.--') ;
            leg = horzcat(leg, strcat('val ', opts.errorLabels)) ;
            set(legend(leg{:}),'color','none') ;
            grid on ;
            xlabel('training epoch') ; ylabel('error') ;
            title('error') ;
        end
        drawnow ;
        
        if mod(epoch - 1, interval) == 0 || epoch == opts.numEpochs
            if mod(epoch - 1, interval) == 0 || epoch == opts.numEpochs
                
                model_file = [opts.working_folder, 'iter_',...
                    num2str(epoch) ,'_model.mat'];
                save(model_file, 'net1', 'net2', 'net3', 'net4', 'info', 'opts');
            end
        end
        print(1, [opts.working_folder, 'net-train.pdf'], '-dpdf') ;
    end
end
end

function net = initialize_momentum(net)
for i=1:numel(net.layers)
    if isfield(net.layers{i}, 'weights')
        J = numel(net.layers{i}.weights) ;
        for j=1:J
            net.layers{i}.momentum{j} = zeros(size(net.layers{i}.weights{j}), 'single') ;
        end
        if ~isfield(net.layers{i}, 'learningRate')
            net.layers{i}.learningRate = ones(1, J, 'single') ;
        end
        if ~isfield(net.layers{i}, 'weightDecay')
            net.layers{i}.weightDecay = ones(1, J, 'single') ;
        end
    end
end
end

function err = error_multiclass(opts, labels, res)
% -------------------------------------------------------------------------
predictions = gather(res(end-1).x) ;
[~,predictions] = sort(predictions, 3, 'descend') ;

% be resilient to badly formatted labels
if numel(labels) == size(predictions, 4)
  labels = reshape(labels,1,1,1,[]) ;
end

% skip null labels
mass = single(labels(:,:,1,:) > 0) ;
if size(labels,3) == 2
  % if there is a second channel in labels, used it as weights
  mass = mass .* labels(:,:,2,:) ;
  labels(:,:,2,:) = [] ;
end

error = ~bsxfun(@eq, predictions, labels) ;
err(1,1) = sum(sum(sum(mass .* error(:,:,1,:)))) ;
err(2,1) = sum(sum(sum(mass .* min(error(:,:,1:5,:),[],3)))) ;
end