function [net, opts] = train_model_multigrid(opts, net, imdb, getBatch)

rate_list = logspace(-2, -4, 80)*100;
learningRate_array = repmat(rate_list , max(1,floor(opts.numEpochs / length(rate_list))),1); %logspace(-2, -4, 60) ones(1,60, 'single')
learningRate_array = reshape(learningRate_array, 1, []);


opts.train = [] ;
opts.val = [] ;
opts.conserveMemory = true ;
opts.backPropDepth = +inf ;
opts.sync = false ;
opts.prefetch = false;
opts.numFetchThreads = 8;
opts.cudnn = true ;
opts.weightDecay = 0.0001 ; %0.0001
opts.momentum = 0.9;
opts.learningRate = reshape(learningRate_array, 1, []);

if isempty(opts.train), opts.train = find(imdb.images.set==1) ; end
if isempty(opts.val), opts.val = find(imdb.images.set==2) ; end
if isnan(opts.train), opts.train = [] ; end

opts.batch_size = min(opts.batch_size, numel(opts.train));

net = initialize_momentum(net);

interval = 10;
SSD = zeros(opts.numEpochs, 1);

mean_img = net.normalization.averageImage;

batchStart = 1;
batchEnd = min(1+opts.batch_size-1, numel(opts.train)) ;
batch = opts.train(batchStart : batchEnd) ;
im = getBatch(imdb, batch);
draw_figures(opts, im, 0, mean_img, [], 'original');

% -------------------------------------------------------------------------
%                           Train and validate
% -------------------------------------------------------------------------

model_file = [opts.working_folder, 'iter_',...
    num2str(opts.numEpochs) ,'_model.mat'];

if exist(model_file, 'file') && opts.force_learn == false;
    load(model_file);
else
    h = figure;
    for epoch=1:opts.numEpochs
        fprintf('Iteration %d / %d\n', epoch, opts.numEpochs);
        
        learningRate = opts.learningRate(min(epoch, numel(opts.learningRate)));
        train = opts.train;
        
        [net, syn_mats, loss] = process_epoch_multigrid2_4by4(opts, getBatch, epoch, train, learningRate, imdb, net);

        SSD(epoch) = loss;
        
        disp(['Loss: ', num2str(SSD(epoch))]);
        if mod(epoch - 1, interval) == 0 || epoch == opts.numEpochs
            
            syn_mat = syn_mats{1};
            draw_figures(opts, syn_mat, epoch, mean_img, SSD, 'recovered');
            
            if mod(epoch - 1, interval) == 0 || epoch == opts.numEpochs
                
                model_file = [opts.working_folder, 'iter_',...
                    num2str(epoch) ,'_model.mat'];
                save(model_file, 'net', 'opts');
                
                % saveas(h, [opts.working_folder, 'iter_',...
                %    num2str(epoch) ,'_error.fig']);
                saveas(h, [opts.working_folder, 'iter_',...
                    num2str(epoch) ,'_error.png'])
            end
        end
    end
end
end

function net = initialize_momentum(net)
for i=1:numel(net.layers)
    if isfield(net.layers{i}, 'weights')
        J = numel(net.layers{i}.weights) ;
        for j=1:J
            net.layers{i}.momentum{j} = zeros(size(net.layers{i}.weights{j}), 'single') ;
        end
        if ~isfield(net.layers{i}, 'learningRate')
            net.layers{i}.learningRate = ones(1, J, 'single') ;
        end
        if ~isfield(net.layers{i}, 'weightDecay')
            net.layers{i}.weightDecay = ones(1, J, 'single') ;
        end
    end
end
end