function [net, config] = learn_multigrid(category, grid)

rng(1);

learningTime = tic;

fprintf('Learning category: %s\n', category);

if nargin < 2
    grid = '16to64';
end
config = frame_config(category, grid);

%% Setup Network 1
switch grid
    case '16to64'
        net = convNet_object_16to64(config);
    case '4to16'
        net = convNet_object_4to16(config);
end

net.normalization.imageSize = [config.sx, config.sy, 3];
net.normalization.averageImage = zeros(net.normalization.imageSize, 'single');

img = randn(net.normalization.imageSize, 'single');
net = vl_simplenn_move(net, 'gpu') ;
res = vl_simplenn(net, gpuArray(img));
net = vl_simplenn_move(net, 'cpu');
config.dydz_sz = size(res(end).x);

net.numFilters = zeros(1, length(net.layers));
for l = 1:length(net.layers)
    if isfield(net.layers{l}, 'weights')
        sz = size(res(l+1).x);
        net.numFilters(l) = sz(1) * sz(2);
    end
end

config.layer_sets = numel(net.layers):-1:1;

clear res;
clear img;

%% Step 2 create imdb
[imdb, getBatch, net] = create_imdb(config, net);

%% Step 3: training
[net, config] = train_model_multigrid(config, net, imdb, getBatch);


learningTime = toc(learningTime);
hrs = floor(learningTime / 3600);
learningTime = mod(learningTime, 3600);
mins = floor(learningTime / 60);
secds = mod(learningTime, 60);
fprintf('total learning time is %d hours / %d minutes / %.2f seconds.\n', hrs, mins, secds);